<div class="container">
    <h1 class="text-center">Object Popup Plugin DEBUG TEST Page</h1>

    Enter the Item Key
    <input [(ngModel)]="itemKey">

    <div>
        Test tooltip
    </div>

    <div>
        Test summary
    </div>

    <div>
        Test details
    </div>
</div>

<div class="peek-tables-section">
    <table class="table">
        <thead>
        <tr class="tr">
            <th class="th">{{doc.key}}</th>
            <th class="th">{{docTypeName}}</th>
        </tr>
        </thead>
        <tbody>
        <tr *ngFor="let item of docProps" class="tr">
            <td class="td">{{item.title}}</td>
            <td class="td">{{item.value}}</td>
        </tr>
        </tbody>
    </table>

</div>