<div class="peek-nav-section">

</div>

<div class="peek-nav-bar-padding ">
    <div class="peek-information-section">
        <button
            (click)="toggleOfflineMode()"
            [class.bg-success]="lastStatus.cacheForOfflineEnabled"
            class="btn"
        >
            {{lastStatus.cacheForOfflineEnabled
            ? 'Disable Offline Support'
            : 'Enable Offline Support'}}
        </button>
        <p>
            Load Complete : {{lastStatus.initialLoadComplete}}<br />
            Load Progress : {{lastStatus.loadProgress}}/{{lastStatus.loadTotal}}<br />
            Last Check : {{lastStatus.lastCheck}}<br />
        </p>
    
    </div>
</div>
