export {
    DocumentResultI
} from "./document-loader/PrivateDocumentLoaderService"

export { SettingPropertyTuple } from "./admin/SettingPropertyTuple"
export { AdminStatusTuple } from "./admin/AdminStatusTuple"
export * from "./PluginNames"

export { OfflineConfigTuple } from "./tuples/OfflineConfigTuple"

export { DocDbTupleService } from "./DocDbTupleService"

