import { CommonModule } from "@angular/common"
import { HttpClientModule } from "@angular/common/http"
import { NgModule } from "@angular/core"
import { NzDropDownModule } from "ng-zorro-antd/dropdown"
import { NzIconModule } from "ng-zorro-antd/icon"
import { NzTableModule } from "ng-zorro-antd/table"
import { NzToolTipModule } from "ng-zorro-antd/tooltip"
import { NzButtonModule } from "ng-zorro-antd/button"
import { NzCardModule } from "ng-zorro-antd/card"
import { NzMenuModule } from "ng-zorro-antd/menu"
import { NzModalModule } from "ng-zorro-antd/modal"

import {
    PopupComponent,
} from "./components"

const COMPONENTS = [
    PopupComponent
]

@NgModule({
    declarations: COMPONENTS,
    imports: [
        CommonModule,
        HttpClientModule,
        NzDropDownModule,
        NzTableModule,
        NzToolTipModule,
        NzButtonModule,
        NzCardModule,
        NzMenuModule,
        NzModalModule,
        NzIconModule,
    ],
    exports: COMPONENTS,
})
export class DocDbPopupModule {
}
